class Puzzle {
	protected int puzzleSize = 9;  // Should always be a square: 4, 9, 16, 25, 36
	protected int sqrtPuzzleSize = 3;

	protected PuzzleCell [] cells;
	protected PuzzleBlock [] blocks;
	protected PuzzleRow [] rows;
	protected PuzzleColumn [] columns;

	public Puzzle()
	{
		cells = new PuzzleCell [puzzleSize * puzzleSize];
		blocks = new PuzzleBlock [puzzleSize];
		rows = new PuzzleRow [puzzleSize];
		columns = new PuzzleColumn [puzzleSize];
	}

	protected void InitializeCells(int [] values) 
	{
		int index = 0;
		for (int v : values) {
			cells[index++] = new PuzzleCell(v);
		}
	}

	protected void InitializeBlocks() 
	{
		for (int cellIndex = 0; cellIndex < cells.length; cellIndex++)  
		{
			int blockRow = cellIndex / (puzzleSize * sqrtPuzzleSize);
			int blockColumn = (cellIndex % puzzleSize) / sqrtPuzzleSize;
			int blockIndex = blockRow * sqrtPuzzleSize + blockColumn;
			if (null == blocks[blockIndex]) 
			{
				blocks[blockIndex] = new PuzzleBlock();
			}
			blocks[blockIndex].AddCell(cells[cellIndex]);
		}
	}

	protected void InitializeRows() 
	{
		for (int cellIndex = 0;  cellIndex < cells.length;  cellIndex++)
		{
			int rowIndex = cellIndex / puzzleSize;
			if (null == rows[rowIndex])
			{
				rows[rowIndex] = new PuzzleRow();
			}
			rows[rowIndex].AddCell(cells[cellIndex]);
		}
	}

	protected void InitializeColumns()
	{
		for (int cellIndex = 0;  cellIndex < cells.length;  cellIndex++)
		{
			int columnIndex = cellIndex % puzzleSize;
			if (null == columns[columnIndex])
			{
				columns[columnIndex] = new PuzzleColumn();
			}
			columns[columnIndex].AddCell(cells[cellIndex]);
		}
	}

	public void Initialize(int [] values) 
	{
		if (values.length == cells.length) 
		{
			InitializeCells(values);
			InitializeBlocks();
			InitializeRows();
			InitializeColumns();
		}
	}

	public boolean isSolved()
	{
		for (PuzzleCell c : cells)
		{
			if (!c.isCertain())
				return false;
		}
		return true;
	}

	public void Advance() 
	{
		AdvancePush();
		AdvancePull();
	}

	public void AdvancePush()
	{
		for (PuzzleCell c : cells)
		{
			if (c.isCertain()) 
			{
				c.getRow().Remove(c);
				c.getColumn().Remove(c);
				c.getBlock().Remove(c);
			}
		}
	}

	public void AdvancePull()
	{
		for (int values = 1;  values < 10;  values++) 
		{
			for (PuzzleRow row : rows)
			{
				row.Pull(values);
			}

			for (PuzzleColumn column : columns)
			{
				column.Pull(values);
			}

			for (PuzzleBlock block : blocks)
			{
				block.Pull(values);
			}
		}
	}

}